///////////////////////////////
//GammaLUT generator
//by Thomas Happ
///////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

///////////////////////////////////////
//This outputs a table for gamma correction.
///////////////////////////////////////
void main () {
	FILE* outFile;
	char* filename = "GammaLUT.h";
	if (!(outFile = fopen(filename, "wt"))) {
		printf("Could not open %s for writing.", filename);
		return;
	}
	fprintf(outFile, "#ifndef GAMMALUT_H\n");
	fprintf(outFile, "#define GAMMALUT_H\n");


	//Print out instructions
	fprintf(outFile, "\n\n//This is a gamma correction lookup table.\n");
	fprintf(outFile, "//Reference it using the equation \"(color << 8) + gamma\",\n");
	fprintf(outFile, "//where gamma is an 8 bit 4/4 fixed point number.\n\n");

	//Print the structure name.
	fprintf(outFile, "extern const unsigned char GammaLUT [8192] __attribute__ ((aligned (4))) = {");
	//Loop through every possible color value and every available gamma, perform correction
	//Does NOT allow a gamma offset
	for (int color = 0; color <= 0x1F; color++) {
		//Loop through gamma (fixed 4/4 fixed point)
		for (int gamma = 0; gamma <= 0xFF; gamma++) {

			double dcolor = color;
			double dgamma = (gamma & 0xF);	//Get the decimal
			dgamma = dgamma / 16;			//divide by 16
			dgamma = dgamma + (double)(gamma >> 4);
			dcolor = pow( (dcolor/31.0), 1/dgamma)*31.0;

			//The following is what the "real" equation should be.  It includes
			//an offset value that we don't have space for in our LUT.
			//dcolor = pow( (1-doffset)*(dcolor/31.0)+doffset, 1/dgamma)*31.0;
			int outColor = dcolor;
			
			//Make sure we stay in bounds
			if (outColor > 0x1F)
				outColor = 0x1F;
			
			if ( ! (gamma & 0xF) )		//Every 16 values,
				fprintf(outFile, "\n\t");	//print a newline.
			fprintf(outFile, "%d, ", outColor);
						
		}
	}
	fprintf(outFile, "\n};\n");
	fprintf(outFile, "#endif\n\n");
		
	
}